@@ -39,6 +39,8 @@ class LibTiffTestCase(PillowTestCase):
         out = self.tempfile("temp.png")
         im.save(out)
 
+        out_bytes = io.BytesIO()
+        im.save(out_bytes, format='tiff', compression='group4')
 
 class TestFileLibTiff(LibTiffTestCase):
 
@@ -170,7 +172,8 @@ class TestFileLibTiff(LibTiffTestCase):
                                 'RowsPerStrip',
                                 'StripOffsets']
             for field in requested_fields:
-                self.assertTrue(field in reloaded, "%s not in metadata" % field)
+                self.assertTrue(field in reloaded,
+                                "%s not in metadata" % field)
 
     def test_additional_metadata(self):
         # these should not crash. Seriously dummy data, most of it doesn't make
@@ -179,11 +182,12 @@ class TestFileLibTiff(LibTiffTestCase):
 
         # Get the list of the ones that we should be able to write
 
-        core_items = dict((tag, info) for tag, info in [(s, TiffTags.lookup(s)) for s
-                                                        in TiffTags.LIBTIFF_CORE]
-                          if info.type is not None)
+        core_items = {tag: info for tag, info in ((s, TiffTags.lookup(s)) for s
+                                                  in TiffTags.LIBTIFF_CORE)
+                      if info.type is not None}
 
-        # Exclude ones that have special meaning that we're already testing them
+        # Exclude ones that have special meaning
+        # that we're already testing them
         im = Image.open('Tests/images/hopper_g4.tif')
         for tag in im.tag_v2.keys():
             try:
@@ -397,6 +401,19 @@ class TestFileLibTiff(LibTiffTestCase):
 
         TiffImagePlugin.READ_LIBTIFF = False
 
+    def test_multipage_nframes(self):
+        # issue #862
+        TiffImagePlugin.READ_LIBTIFF = True
+        im = Image.open('Tests/images/multipage.tiff')
+        frames = im.n_frames
+        self.assertEqual(frames, 3)
+        for idx in range(frames):
+            im.seek(0)
+            # Should not raise ValueError: I/O operation on closed file
+            im.load()
+
+        TiffImagePlugin.READ_LIBTIFF = False
+
     def test__next(self):
         TiffImagePlugin.READ_LIBTIFF = True
         im = Image.open('Tests/images/hopper.tif')
@@ -422,8 +439,8 @@ class TestFileLibTiff(LibTiffTestCase):
     def test_gray_semibyte_per_pixel(self):
         test_files = (
             (
-                24.8,#epsilon
-                (#group
+                24.8,  # epsilon
+                (  # group
                     "Tests/images/tiff_gray_2_4_bpp/hopper2.tif",
                     "Tests/images/tiff_gray_2_4_bpp/hopper2I.tif",
                     "Tests/images/tiff_gray_2_4_bpp/hopper2R.tif",
@@ -431,8 +448,8 @@ class TestFileLibTiff(LibTiffTestCase):
                 )
             ),
             (
-                7.3,#epsilon
-                (#group
+                7.3,  # epsilon
+                (  # group
                     "Tests/images/tiff_gray_2_4_bpp/hopper4.tif",
                     "Tests/images/tiff_gray_2_4_bpp/hopper4I.tif",
                     "Tests/images/tiff_gray_2_4_bpp/hopper4R.tif",
@@ -503,9 +520,63 @@ class TestFileLibTiff(LibTiffTestCase):
         # Should not divide by zero
         im.save(outfile)
 
+    def test_fd_duplication(self):
+        # https://github.com/python-pillow/Pillow/issues/1651
+
+        tmpfile = self.tempfile("temp.tif")
+        with open(tmpfile, 'wb') as f:
+            with open("Tests/images/g4-multi.tiff", 'rb') as src:
+                f.write(src.read())
+
+        im = Image.open(tmpfile)
+        count = im.n_frames
+        im.close()
+        try:
+            os.remove(tmpfile) # Windows PermissionError here!
+        except:
+            self.fail("Should not get permission error here")
+
+    def test_read_icc(self):
+        with Image.open("Tests/images/hopper.iccprofile.tif") as img:
+            icc = img.info.get('icc_profile')
+            self.assertNotEqual(icc, None)
+        TiffImagePlugin.READ_LIBTIFF = True
+        with Image.open("Tests/images/hopper.iccprofile.tif") as img:
+            icc_libtiff = img.info.get('icc_profile')
+            self.assertNotEqual(icc_libtiff, None)
+        TiffImagePlugin.READ_LIBTIFF = False
+        self.assertEqual(icc, icc_libtiff)
+
+    def test_multipage_compression(self):
+        im = Image.open('Tests/images/compression.tif')
+
+        im.seek(0)
+        self.assertEqual(im._compression, 'tiff_ccitt')
+        self.assertEqual(im.size, (10, 10))
+
+        im.seek(1)
+        self.assertEqual(im._compression, 'packbits')
+        self.assertEqual(im.size, (10, 10))
+        im.load()
+
+        im.seek(0)
+        self.assertEqual(im._compression, 'tiff_ccitt')
+        self.assertEqual(im.size, (10, 10))
+        im.load()
+
+    def test_save_tiff_with_jpegtables(self):
+        # Arrange
+        outfile = self.tempfile("temp.tif")
+
+        # Created with ImageMagick: convert hopper.jpg hopper_jpg.tif
+        # Contains JPEGTables (347) tag
+        infile = "Tests/images/hopper_jpg.tif"
+        im = Image.open(infile)
+
+        # Act / Assert
+        # Should not raise UnicodeDecodeError or anything else
+        im.save(outfile)
 
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
